#!/usr/bin/perl

use strict;
my $geofile;

sub read_string {
    my $amount = shift || 1;
    my $buffer = '';
    read( $geofile, $buffer, $amount );
    return unpack( "a$amount", $buffer );
}

sub read_int {
    my $amount = shift || 1;
    my $buffer = '';
    read( $geofile, $buffer, 4 * $amount );
    return unpack( "I$amount", $buffer );
}

sub read_float {
    my $amount = shift || 1;
    my $buffer = '';
    read( $geofile, $buffer, 4 * $amount );
    return unpack( "f$amount", $buffer );
}

sub read_short {
    my $amount = shift || 1;
    my $buffer = '';
    read( $geofile, $buffer, 2 * $amount );
    my @shorts = unpack( "s$amount", $buffer );

    print join(',' ,@shorts) . ':';
    return @shorts;
}

sub read_geo {
    my $filename = shift || warn("dumpgeo.pl 0.1 - Dump Horde3D geo data by nieka\@daansystems.com.\nUsage: dumpgeo.pl <geofile>\n");
    open( $geofile, "<$filename" )
      || die("cannot read $filename\n");
    binmode($geofile);

    warn "reading from: $filename\n";
    my $buffer = '';
    print 'header: ' . read_string(4) . "\n";    # header
    print 'file version: ' . read_int() . "\n";
    print 'num joints: ' . ( read_int() - 1 ) . "\n";    # numjoints (0+1)
                                                         # identity matrix
    print 'default matrix: ' . join( ',', read_float(16) ) . "\n";

    my $numstreams = read_int();
    print 'num vertex streams: ' . $numstreams . "\n";
    my $numpos = read_int();
    print 'num positions: ' . $numpos . "\n";

    my %magic = (
        0 => 'vertices',
        1 => 'normals',
        2 => 'tangents',
        3 => 'bitangents',
        6 => 'tcoords1',
        7 => 'tcoords2',

    );

    print 'magic: ' . $magic{ read_int() } . "\n";
    print 'stream element size: ' . read_int() . "\n";

    for ( 1 .. $numpos ) {
        print join( ',', read_float(3) ) . "\n";
    }

    print 'magic: ' . $magic{ read_int() } . "\n";
    print 'stream element size: ' . read_int() . "\n";

    for ( 1 .. $numpos ) {
        print join( ',', map { $_ / 32767} read_short(3) ) . "\n";
    }

    print 'magic: ' . $magic{ read_int() } . "\n";
    print 'stream element size: ' . read_int() . "\n";

    for ( 1 .. $numpos ) {
        print join( ',', map { $_ / 32767} read_short(3) ) . "\n";
    }

    print 'magic: ' . $magic{ read_int() } . "\n";
    print 'stream element size: ' . read_int() . "\n";

    for ( 1 .. $numpos ) {
        print join( ',', map { $_ / 32767} read_short(3) ) . "\n";
    }

    print 'magic: ' . $magic{ read_int() } . "\n";
    print 'stream element size: ' . read_int() . "\n";

    for ( 1 .. $numpos ) {
        print join( ',', read_float(2) ) . "\n";
    }

    print 'magic: ' . $magic{ read_int() } . "\n";
    print 'stream element size: ' . read_int() . "\n";

    for ( 1 .. $numpos ) {
        print join( ',',  read_float(2) ) . "\n";
    }

    my $numindices = read_int();
    print 'num indices: ' . $numindices . "\n";

    for ( 1 .. $numindices ) {
        print join( ',', read_int(1) ) . "\n";
    }

    print 'num morph targets: ' . read_int() . "\n";
    close($geofile);

}

read_geo(@ARGV);
